/*
 */

#include "chiark-tcl-base.h"

int cht_staticerr(Tcl_Interp *ip, const char *m, const char *ec) {
  Tcl_SetResult(ip, (char*)m, TCL_STATIC);
  if (ec) Tcl_SetObjErrorCode(ip, Tcl_NewStringObj(ec,-1));
  return TCL_ERROR;
}

int cht_posixerr(Tcl_Interp *ip, int errnoval, const char *m) {
  const char *em;
  
  Tcl_ResetResult(ip);
  errno= errnoval;
  em= Tcl_PosixError(ip);
  Tcl_AppendResult(ip, m, ": ", em, (char*)0);
  return TCL_ERROR;
}

int cht_newfdposixerr(Tcl_Interp *ip, int fd, const char *m) {
  int e;
  e= errno;
  close(fd);
  return cht_posixerr(ip,e,m);
}

void cht_objfreeir(Tcl_Obj *o) {
  if (o->typePtr && o->typePtr->freeIntRepProc)
    o->typePtr->freeIntRepProc(o);
  o->typePtr= 0;
}  

void cht_obj_updatestr_vstringls(Tcl_Obj *o, ...) {
  va_list al;
  char *p;
  const char *part;
  int l, pl;

  va_start(al,o);
  for (l=0; (part= va_arg(al, const char*)); )
    l+= va_arg(al, int);
  va_end(al);
  
  o->length= l;
  o->bytes= TALLOC(l+1);

  va_start(al,o);
  for (p= o->bytes; (part= va_arg(al, const char*)); p += pl) {
    pl= va_arg(al, int);
    memcpy(p, part, pl);
  }
  va_end(al);

  *p= 0;
}

void cht_obj_updatestr_string(Tcl_Obj *o, const char *str) {
  cht_obj_updatestr_vstringls(o, str, strlen(str), (char*)0);
}

#define URANDOM "/dev/urandom"

int cht_get_urandom(Tcl_Interp *ip, Byte *buffer, int l) {
  static FILE *urandom;

  int r, esave;

  if (!urandom) {
    urandom= fopen(URANDOM,"rb");
    if (!urandom) return cht_posixerr(ip,errno,"open " URANDOM);
  }
  r= fread(buffer,1,l,urandom);
  if (r==l) return 0;

  esave= errno;
  fclose(urandom); urandom=0;

  if (ferror(urandom)) {
    return cht_posixerr(ip,errno,"read " URANDOM);
  } else {
    assert(feof(urandom));
    return cht_staticerr(ip, URANDOM " gave eof!", 0);
  }
}

int cht_initextension(Tcl_Interp *ip, const TopLevel_Command *cmds,
			int *donep /* or 0, meaning no types follow */,
			... /* types, terminated by 0 */) {
  static int cht_initd;

  const TopLevel_Command *cmd;
  Tcl_ObjType *ot;

  va_list al;

  if (!cht_initd) {
    cht_initd= 1;
    Tcl_RegisterObjType(&cht_tabledataid_nearlytype);
    Tcl_RegisterObjType(&cht_enum_nearlytype);
    Tcl_RegisterObjType(&cht_enum1_nearlytype);
  }

  if (donep && !*donep) {
    *donep= 1;
    va_start(al, donep);
    while ((ot= va_arg(al, Tcl_ObjType*)))
      Tcl_RegisterObjType(ot);
  }

  for (cmd= cmds;
       cmd->name;
       cmd++)
    Tcl_CreateObjCommand(ip, (char*)cmd->name, cmd->func, 0,0);

  return TCL_OK;
}
